#!/usr/bin/perl
=head1 NAME

gen-mods-list.pl - a script to gather list of packages require'd or use'd
directly, or list of packages exported by the Perl code.

=head1 SYNOPSIS

B<gen-mods-list.pl> S<[ B<-h> ] > |
    S<[ B<-l|--listpackages> ] [ B<-p|--perl> ] I<srcfiles>..>

=head1 DESCRIPTION

This script extracts either names of packages directly require'd or use'd
by the set of Perl source files, or (if B<-l> commandline option specified)
names of packages defined by  the set of Perl source files. The names of
packages are printed to stdout, each line being either just name of package
(e.g. C<Getopt::Long>) or (if B<-p> commandline option specified) a 
full perl C<use> statement (e.g. C<{ use Getopt::Long; }>).

This script is useful for packing Perl code protected using Stunnix Perl-Obfus
into standalone executable using tools like PAR toolkit or like Perl2Exe; 
such tools need a way to compute set of modules the code depends on - and 
since they fail to compute the set of dependencies of protected code, 
this utility should be used to extract names of all dependencies. Most 
recommended approach is extracting names of dependencies using B<-p>
commandline switch - it will produce perl program that does nothing but
use's all dependencies; such produced perl should be included as-is along your
protected code. Stunnix Perl Web Server and Project Manager were packaged as 
standalone executable using exactly this approach.

=head1 OPTIONS

=over 4

=item B<-l>

Instructs to gather names of packages defined by the set of files (by use
of C<package> keyword only) rather than names of packages require'd or use'd.

=item B<-p>

Instructs to output list of packages as a set of 'use' statements, e.g.
C<{ use Getopt::Long;}>


=back 

=head1 NOTES

Perl 5.8.x is required to run this utility; Perl 5.6.x and below are not
supported.


=cut
die('this is a stub of get-mods-list.pl. To evaluate a product, ask Stunnix support for a real trial via email');
